package info.lotosummary.software;
import java.io.*;
import java.util.*;

public class GapID {
    static final char COMPLEMENTARY = '1';
    static final char GAP_C = ' ';
    static final double N_2_31 = 2147483648.0; // 2^31 = 2147483648
    static final int C_ARRAY_SIZE = 64;
    static final int FROM = 900;
    static final int ID_SIZE = 8;
    static final int NOT_CONVERTED = -1;
    static final String EMPTY = "";
    static final String GAP = " ";
    static final String[] C = {"/", "3", "b", "E", "6", "K", "y", "c", "h", "z", "J", "7", "g", "p", "Y", "o", "5", "w", "F", "I", "1", "G", "s", "A", "P", "L", "2", "T", "X", "9", "V", "x", "j", "+", "R", "H", "d", "M", "e", "N", "q", "Z", "D", "C", "v", "n", "B", "8", "k", "f", "W", "U", "0", "S", "O", "4", "u", "Q", "a", "t", "r", "i", "m", "l"};
    static final String RANNU_TEXT = "rannu.txt";

    int session;
    int[] random_number;
    int[][] two_digit_number;
    long table_size;
    PRNG prn;
    String error;

    private boolean readRannuText() {
        int c0, c1, lineLength;
        int k = 0;
        int file_size;
        String line;

// size of RANNU_TEXT
        File test = new File(RANNU_TEXT);
        if( test.exists() ) {
            file_size = (int)test.length();
        } else {
            error = RANNU_TEXT;
            return false;
        }

// random-number table
        random_number = new int[ file_size/3 ];

        try {
            BufferedReader file = new BufferedReader( new FileReader(RANNU_TEXT) );
            for(;;) {
                line = file.readLine();
                if(line == null) {
                    break;
                }

                lineLength = line.length();
                for(int b = 0; b < lineLength; ++b) {
                    c0 = line.charAt(b);
                    if(c0 == GAP_C) {
                        continue;
                    }

                    ++b;
                    if(b < lineLength) {
                        c1 = line.charAt(b);
                        if(c1 == GAP_C) {
                            c1 = COMPLEMENTARY;
                        }

                        random_number[k++] = two_digit_number[c0][c1];
                    } else {
                        break;
                    }

                }

            }

            file.close();
            table_size = k;
            return true;
        } catch (IOException e63) {
            error = "IOException e63: " + RANNU_TEXT;
            return false;
        }

    }

    public GapID() {
        two_digit_number = new int[0x100][0x100];
        two_digit_number['0']['0'] = 0; two_digit_number['0']['1'] = 1; two_digit_number['0']['2'] = 2; two_digit_number['0']['3'] = 3; two_digit_number['0']['4'] = 4; two_digit_number['0']['5'] = 5; two_digit_number['0']['6'] = 6; two_digit_number['0']['7'] = 7; two_digit_number['0']['8'] = 8; two_digit_number['0']['9'] = 9; 
        two_digit_number['1']['0'] = 10; two_digit_number['1']['1'] = 11; two_digit_number['1']['2'] = 12; two_digit_number['1']['3'] = 13; two_digit_number['1']['4'] = 14; two_digit_number['1']['5'] = 15; two_digit_number['1']['6'] = 16; two_digit_number['1']['7'] = 17; two_digit_number['1']['8'] = 18; two_digit_number['1']['9'] = 19; 
        two_digit_number['2']['0'] = 20; two_digit_number['2']['1'] = 21; two_digit_number['2']['2'] = 22; two_digit_number['2']['3'] = 23; two_digit_number['2']['4'] = 24; two_digit_number['2']['5'] = 25; two_digit_number['2']['6'] = 26; two_digit_number['2']['7'] = 27; two_digit_number['2']['8'] = 28; two_digit_number['2']['9'] = 29; 
        two_digit_number['3']['0'] = 30; two_digit_number['3']['1'] = 31; two_digit_number['3']['2'] = 32; two_digit_number['3']['3'] = 33; two_digit_number['3']['4'] = 34; two_digit_number['3']['5'] = 35; two_digit_number['3']['6'] = 36; two_digit_number['3']['7'] = 37; two_digit_number['3']['8'] = 38; two_digit_number['3']['9'] = 39; 
        two_digit_number['4']['0'] = 40; two_digit_number['4']['1'] = 41; two_digit_number['4']['2'] = 42; two_digit_number['4']['3'] = 43; two_digit_number['4']['4'] = 44; two_digit_number['4']['5'] = 45; two_digit_number['4']['6'] = 46; two_digit_number['4']['7'] = 47; two_digit_number['4']['8'] = 48; two_digit_number['4']['9'] = 49; 
        two_digit_number['5']['0'] = 50; two_digit_number['5']['1'] = 51; two_digit_number['5']['2'] = 52; two_digit_number['5']['3'] = 53; two_digit_number['5']['4'] = 54; two_digit_number['5']['5'] = 55; two_digit_number['5']['6'] = 56; two_digit_number['5']['7'] = 57; two_digit_number['5']['8'] = 58; two_digit_number['5']['9'] = 59; 
        two_digit_number['6']['0'] = 60; two_digit_number['6']['1'] = 61; two_digit_number['6']['2'] = 62; two_digit_number['6']['3'] = 63; two_digit_number['6']['4'] = 64; two_digit_number['6']['5'] = 65; two_digit_number['6']['6'] = 66; two_digit_number['6']['7'] = 67; two_digit_number['6']['8'] = 68; two_digit_number['6']['9'] = 69; 
        two_digit_number['7']['0'] = 70; two_digit_number['7']['1'] = 71; two_digit_number['7']['2'] = 72; two_digit_number['7']['3'] = 73; two_digit_number['7']['4'] = 74; two_digit_number['7']['5'] = 75; two_digit_number['7']['6'] = 76; two_digit_number['7']['7'] = 77; two_digit_number['7']['8'] = 78; two_digit_number['7']['9'] = 79; 
        two_digit_number['8']['0'] = 80; two_digit_number['8']['1'] = 81; two_digit_number['8']['2'] = 82; two_digit_number['8']['3'] = 83; two_digit_number['8']['4'] = 84; two_digit_number['8']['5'] = 85; two_digit_number['8']['6'] = 86; two_digit_number['8']['7'] = 87; two_digit_number['8']['8'] = 88; two_digit_number['8']['9'] = 89; 
        two_digit_number['9']['0'] = 90; two_digit_number['9']['1'] = 91; two_digit_number['9']['2'] = 92; two_digit_number['9']['3'] = 93; two_digit_number['9']['4'] = 94; two_digit_number['9']['5'] = 95; two_digit_number['9']['6'] = 96; two_digit_number['9']['7'] = 97; two_digit_number['9']['8'] = 98; two_digit_number['9']['9'] = 99; 

// Time is used as seed.
        Calendar now = Calendar.getInstance();
        int year = now.get(Calendar.YEAR);
        int month = now.get(Calendar.MONTH);
        int day = now.get(Calendar.DATE);
        int hour = now.get(Calendar.HOUR);
        int minute = now.get(Calendar.MINUTE);
        int second = now.get(Calendar.SECOND);
        int add = minute * 60 + hour * 60 * 60;
        int year_month_day_43 = (year + month + day) * 43;
        second += add;
        second += year_month_day_43;

        long seed = (long)second;
        prn = new MT(seed);

// idling
        for(int i = 0; i < FROM; ++i) { 
            prn.nextRandom();
        }

    }

    public long tableSize() {
        return table_size;
    }

    public String inquiry() {
        return error;
    }

    public String gapID() {
// Reading RANNU_TEXT to produce a random-number array
        if(! readRannuText() ) {
            return EMPTY;
        }

// Randomly selecting u so that the array index starts from u.
        int interval_0 = 0;
        int interval_1 = (int)table_size - 1;
        double difference = (double)( interval_1 - interval_0 + 1 );
        double range = difference/4294967296.0;
        double g = (double)prn.nextRandom();
        g += N_2_31; // 0 to 4294967295

        int plus = (int)Math.floor( range * g );
        int u = interval_0 + plus;
        u += session;

// Producing new ID
        int b;
        StringBuffer os = new StringBuffer();
        os.append(" ID:");
        for(int i = 0; i < ID_SIZE; ++i) {
            b = random_number[u++];
            if(u >= table_size) {
                u = 0;
            }

            if(b >= C_ARRAY_SIZE) {
                b = C_ARRAY_SIZE - 1;
            }

            os.append(C[b]);
        }

        String s = os.toString();
        session += 1;
        return s;
    }

}
